from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.request import Request
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class RequestsPaginatedList:
    """  """

    _next_token: Union[Unset, str] = UNSET
    _requests: Union[Unset, List[Request]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("requests={}".format(repr(self._requests)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        next_token = self._next_token
        requests: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._requests, Unset):
            requests = []
            for requests_item_data in self._requests:
                requests_item = requests_item_data.to_dict()

                requests.append(requests_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token
        if requests is not UNSET:
            field_dict["requests"] = requests

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        def get_requests() -> Union[Unset, List[Request]]:
            requests = []
            _requests = d.pop("requests")
            for requests_item_data in _requests or []:
                requests_item = Request.from_dict(requests_item_data, strict=False)

                requests.append(requests_item)

            return requests

        try:
            requests = get_requests()
        except KeyError:
            if strict:
                raise
            requests = cast(Union[Unset, List[Request]], UNSET)

        requests_paginated_list = cls(
            next_token=next_token,
            requests=requests,
        )

        requests_paginated_list.additional_properties = d
        return requests_paginated_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET

    @property
    def requests(self) -> List[Request]:
        if isinstance(self._requests, Unset):
            raise NotPresentError(self, "requests")
        return self._requests

    @requests.setter
    def requests(self, value: List[Request]) -> None:
        self._requests = value

    @requests.deleter
    def requests(self) -> None:
        self._requests = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.workflow_task_status import WorkflowTaskStatus
from ..models.workflow_task_status_lifecycle_transition import WorkflowTaskStatusLifecycleTransition
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskStatusLifecycle")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskStatusLifecycle:
    """  """

    _id: Union[Unset, str] = UNSET
    _initial_status: Union[Unset, WorkflowTaskStatus] = UNSET
    _name: Union[Unset, str] = UNSET
    _statuses: Union[Unset, List[WorkflowTaskStatus]] = UNSET
    _transitions: Union[Unset, List[WorkflowTaskStatusLifecycleTransition]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("initial_status={}".format(repr(self._initial_status)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("statuses={}".format(repr(self._statuses)))
        fields.append("transitions={}".format(repr(self._transitions)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskStatusLifecycle({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        initial_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._initial_status, Unset):
            initial_status = self._initial_status.to_dict()

        name = self._name
        statuses: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._statuses, Unset):
            statuses = []
            for statuses_item_data in self._statuses:
                statuses_item = statuses_item_data.to_dict()

                statuses.append(statuses_item)

        transitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._transitions, Unset):
            transitions = []
            for transitions_item_data in self._transitions:
                transitions_item = transitions_item_data.to_dict()

                transitions.append(transitions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if initial_status is not UNSET:
            field_dict["initialStatus"] = initial_status
        if name is not UNSET:
            field_dict["name"] = name
        if statuses is not UNSET:
            field_dict["statuses"] = statuses
        if transitions is not UNSET:
            field_dict["transitions"] = transitions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_initial_status() -> Union[Unset, WorkflowTaskStatus]:
            initial_status: Union[Unset, Union[Unset, WorkflowTaskStatus]] = UNSET
            _initial_status = d.pop("initialStatus")

            if not isinstance(_initial_status, Unset):
                initial_status = WorkflowTaskStatus.from_dict(_initial_status)

            return initial_status

        try:
            initial_status = get_initial_status()
        except KeyError:
            if strict:
                raise
            initial_status = cast(Union[Unset, WorkflowTaskStatus], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_statuses() -> Union[Unset, List[WorkflowTaskStatus]]:
            statuses = []
            _statuses = d.pop("statuses")
            for statuses_item_data in _statuses or []:
                statuses_item = WorkflowTaskStatus.from_dict(statuses_item_data, strict=False)

                statuses.append(statuses_item)

            return statuses

        try:
            statuses = get_statuses()
        except KeyError:
            if strict:
                raise
            statuses = cast(Union[Unset, List[WorkflowTaskStatus]], UNSET)

        def get_transitions() -> Union[Unset, List[WorkflowTaskStatusLifecycleTransition]]:
            transitions = []
            _transitions = d.pop("transitions")
            for transitions_item_data in _transitions or []:
                transitions_item = WorkflowTaskStatusLifecycleTransition.from_dict(
                    transitions_item_data, strict=False
                )

                transitions.append(transitions_item)

            return transitions

        try:
            transitions = get_transitions()
        except KeyError:
            if strict:
                raise
            transitions = cast(Union[Unset, List[WorkflowTaskStatusLifecycleTransition]], UNSET)

        workflow_task_status_lifecycle = cls(
            id=id,
            initial_status=initial_status,
            name=name,
            statuses=statuses,
            transitions=transitions,
        )

        workflow_task_status_lifecycle.additional_properties = d
        return workflow_task_status_lifecycle

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def initial_status(self) -> WorkflowTaskStatus:
        if isinstance(self._initial_status, Unset):
            raise NotPresentError(self, "initial_status")
        return self._initial_status

    @initial_status.setter
    def initial_status(self, value: WorkflowTaskStatus) -> None:
        self._initial_status = value

    @initial_status.deleter
    def initial_status(self) -> None:
        self._initial_status = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def statuses(self) -> List[WorkflowTaskStatus]:
        if isinstance(self._statuses, Unset):
            raise NotPresentError(self, "statuses")
        return self._statuses

    @statuses.setter
    def statuses(self, value: List[WorkflowTaskStatus]) -> None:
        self._statuses = value

    @statuses.deleter
    def statuses(self) -> None:
        self._statuses = UNSET

    @property
    def transitions(self) -> List[WorkflowTaskStatusLifecycleTransition]:
        if isinstance(self._transitions, Unset):
            raise NotPresentError(self, "transitions")
        return self._transitions

    @transitions.setter
    def transitions(self, value: List[WorkflowTaskStatusLifecycleTransition]) -> None:
        self._transitions = value

    @transitions.deleter
    def transitions(self) -> None:
        self._transitions = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectsArchivalChange")


@attr.s(auto_attribs=True, repr=False)
class ProjectsArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of projects along with any IDs of project contents that were unarchived."""

    _aa_sequence_ids: Union[Unset, List[str]] = UNSET
    _custom_entity_ids: Union[Unset, List[str]] = UNSET
    _dna_sequence_ids: Union[Unset, List[str]] = UNSET
    _entry_ids: Union[Unset, List[str]] = UNSET
    _folder_ids: Union[Unset, List[str]] = UNSET
    _mixture_ids: Union[Unset, List[str]] = UNSET
    _oligo_ids: Union[Unset, List[str]] = UNSET
    _project_ids: Union[Unset, List[str]] = UNSET
    _protocol_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("aa_sequence_ids={}".format(repr(self._aa_sequence_ids)))
        fields.append("custom_entity_ids={}".format(repr(self._custom_entity_ids)))
        fields.append("dna_sequence_ids={}".format(repr(self._dna_sequence_ids)))
        fields.append("entry_ids={}".format(repr(self._entry_ids)))
        fields.append("folder_ids={}".format(repr(self._folder_ids)))
        fields.append("mixture_ids={}".format(repr(self._mixture_ids)))
        fields.append("oligo_ids={}".format(repr(self._oligo_ids)))
        fields.append("project_ids={}".format(repr(self._project_ids)))
        fields.append("protocol_ids={}".format(repr(self._protocol_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ProjectsArchivalChange({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aa_sequence_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._aa_sequence_ids, Unset):
            aa_sequence_ids = self._aa_sequence_ids

        custom_entity_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_entity_ids, Unset):
            custom_entity_ids = self._custom_entity_ids

        dna_sequence_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dna_sequence_ids, Unset):
            dna_sequence_ids = self._dna_sequence_ids

        entry_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._entry_ids, Unset):
            entry_ids = self._entry_ids

        folder_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._folder_ids, Unset):
            folder_ids = self._folder_ids

        mixture_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._mixture_ids, Unset):
            mixture_ids = self._mixture_ids

        oligo_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._oligo_ids, Unset):
            oligo_ids = self._oligo_ids

        project_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._project_ids, Unset):
            project_ids = self._project_ids

        protocol_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._protocol_ids, Unset):
            protocol_ids = self._protocol_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aa_sequence_ids is not UNSET:
            field_dict["aaSequenceIds"] = aa_sequence_ids
        if custom_entity_ids is not UNSET:
            field_dict["customEntityIds"] = custom_entity_ids
        if dna_sequence_ids is not UNSET:
            field_dict["dnaSequenceIds"] = dna_sequence_ids
        if entry_ids is not UNSET:
            field_dict["entryIds"] = entry_ids
        if folder_ids is not UNSET:
            field_dict["folderIds"] = folder_ids
        if mixture_ids is not UNSET:
            field_dict["mixtureIds"] = mixture_ids
        if oligo_ids is not UNSET:
            field_dict["oligoIds"] = oligo_ids
        if project_ids is not UNSET:
            field_dict["projectIds"] = project_ids
        if protocol_ids is not UNSET:
            field_dict["protocolIds"] = protocol_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aa_sequence_ids() -> Union[Unset, List[str]]:
            aa_sequence_ids = cast(List[str], d.pop("aaSequenceIds"))

            return aa_sequence_ids

        try:
            aa_sequence_ids = get_aa_sequence_ids()
        except KeyError:
            if strict:
                raise
            aa_sequence_ids = cast(Union[Unset, List[str]], UNSET)

        def get_custom_entity_ids() -> Union[Unset, List[str]]:
            custom_entity_ids = cast(List[str], d.pop("customEntityIds"))

            return custom_entity_ids

        try:
            custom_entity_ids = get_custom_entity_ids()
        except KeyError:
            if strict:
                raise
            custom_entity_ids = cast(Union[Unset, List[str]], UNSET)

        def get_dna_sequence_ids() -> Union[Unset, List[str]]:
            dna_sequence_ids = cast(List[str], d.pop("dnaSequenceIds"))

            return dna_sequence_ids

        try:
            dna_sequence_ids = get_dna_sequence_ids()
        except KeyError:
            if strict:
                raise
            dna_sequence_ids = cast(Union[Unset, List[str]], UNSET)

        def get_entry_ids() -> Union[Unset, List[str]]:
            entry_ids = cast(List[str], d.pop("entryIds"))

            return entry_ids

        try:
            entry_ids = get_entry_ids()
        except KeyError:
            if strict:
                raise
            entry_ids = cast(Union[Unset, List[str]], UNSET)

        def get_folder_ids() -> Union[Unset, List[str]]:
            folder_ids = cast(List[str], d.pop("folderIds"))

            return folder_ids

        try:
            folder_ids = get_folder_ids()
        except KeyError:
            if strict:
                raise
            folder_ids = cast(Union[Unset, List[str]], UNSET)

        def get_mixture_ids() -> Union[Unset, List[str]]:
            mixture_ids = cast(List[str], d.pop("mixtureIds"))

            return mixture_ids

        try:
            mixture_ids = get_mixture_ids()
        except KeyError:
            if strict:
                raise
            mixture_ids = cast(Union[Unset, List[str]], UNSET)

        def get_oligo_ids() -> Union[Unset, List[str]]:
            oligo_ids = cast(List[str], d.pop("oligoIds"))

            return oligo_ids

        try:
            oligo_ids = get_oligo_ids()
        except KeyError:
            if strict:
                raise
            oligo_ids = cast(Union[Unset, List[str]], UNSET)

        def get_project_ids() -> Union[Unset, List[str]]:
            project_ids = cast(List[str], d.pop("projectIds"))

            return project_ids

        try:
            project_ids = get_project_ids()
        except KeyError:
            if strict:
                raise
            project_ids = cast(Union[Unset, List[str]], UNSET)

        def get_protocol_ids() -> Union[Unset, List[str]]:
            protocol_ids = cast(List[str], d.pop("protocolIds"))

            return protocol_ids

        try:
            protocol_ids = get_protocol_ids()
        except KeyError:
            if strict:
                raise
            protocol_ids = cast(Union[Unset, List[str]], UNSET)

        projects_archival_change = cls(
            aa_sequence_ids=aa_sequence_ids,
            custom_entity_ids=custom_entity_ids,
            dna_sequence_ids=dna_sequence_ids,
            entry_ids=entry_ids,
            folder_ids=folder_ids,
            mixture_ids=mixture_ids,
            oligo_ids=oligo_ids,
            project_ids=project_ids,
            protocol_ids=protocol_ids,
        )

        projects_archival_change.additional_properties = d
        return projects_archival_change

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def aa_sequence_ids(self) -> List[str]:
        if isinstance(self._aa_sequence_ids, Unset):
            raise NotPresentError(self, "aa_sequence_ids")
        return self._aa_sequence_ids

    @aa_sequence_ids.setter
    def aa_sequence_ids(self, value: List[str]) -> None:
        self._aa_sequence_ids = value

    @aa_sequence_ids.deleter
    def aa_sequence_ids(self) -> None:
        self._aa_sequence_ids = UNSET

    @property
    def custom_entity_ids(self) -> List[str]:
        if isinstance(self._custom_entity_ids, Unset):
            raise NotPresentError(self, "custom_entity_ids")
        return self._custom_entity_ids

    @custom_entity_ids.setter
    def custom_entity_ids(self, value: List[str]) -> None:
        self._custom_entity_ids = value

    @custom_entity_ids.deleter
    def custom_entity_ids(self) -> None:
        self._custom_entity_ids = UNSET

    @property
    def dna_sequence_ids(self) -> List[str]:
        if isinstance(self._dna_sequence_ids, Unset):
            raise NotPresentError(self, "dna_sequence_ids")
        return self._dna_sequence_ids

    @dna_sequence_ids.setter
    def dna_sequence_ids(self, value: List[str]) -> None:
        self._dna_sequence_ids = value

    @dna_sequence_ids.deleter
    def dna_sequence_ids(self) -> None:
        self._dna_sequence_ids = UNSET

    @property
    def entry_ids(self) -> List[str]:
        if isinstance(self._entry_ids, Unset):
            raise NotPresentError(self, "entry_ids")
        return self._entry_ids

    @entry_ids.setter
    def entry_ids(self, value: List[str]) -> None:
        self._entry_ids = value

    @entry_ids.deleter
    def entry_ids(self) -> None:
        self._entry_ids = UNSET

    @property
    def folder_ids(self) -> List[str]:
        if isinstance(self._folder_ids, Unset):
            raise NotPresentError(self, "folder_ids")
        return self._folder_ids

    @folder_ids.setter
    def folder_ids(self, value: List[str]) -> None:
        self._folder_ids = value

    @folder_ids.deleter
    def folder_ids(self) -> None:
        self._folder_ids = UNSET

    @property
    def mixture_ids(self) -> List[str]:
        if isinstance(self._mixture_ids, Unset):
            raise NotPresentError(self, "mixture_ids")
        return self._mixture_ids

    @mixture_ids.setter
    def mixture_ids(self, value: List[str]) -> None:
        self._mixture_ids = value

    @mixture_ids.deleter
    def mixture_ids(self) -> None:
        self._mixture_ids = UNSET

    @property
    def oligo_ids(self) -> List[str]:
        if isinstance(self._oligo_ids, Unset):
            raise NotPresentError(self, "oligo_ids")
        return self._oligo_ids

    @oligo_ids.setter
    def oligo_ids(self, value: List[str]) -> None:
        self._oligo_ids = value

    @oligo_ids.deleter
    def oligo_ids(self) -> None:
        self._oligo_ids = UNSET

    @property
    def project_ids(self) -> List[str]:
        if isinstance(self._project_ids, Unset):
            raise NotPresentError(self, "project_ids")
        return self._project_ids

    @project_ids.setter
    def project_ids(self, value: List[str]) -> None:
        self._project_ids = value

    @project_ids.deleter
    def project_ids(self) -> None:
        self._project_ids = UNSET

    @property
    def protocol_ids(self) -> List[str]:
        if isinstance(self._protocol_ids, Unset):
            raise NotPresentError(self, "protocol_ids")
        return self._protocol_ids

    @protocol_ids.setter
    def protocol_ids(self, value: List[str]) -> None:
        self._protocol_ids = value

    @protocol_ids.deleter
    def protocol_ids(self) -> None:
        self._protocol_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.barcode_scan_form_instance_provider_identifier_type import (
    BarcodeScanFormInstanceProviderIdentifierType,
)
from ..models.barcode_scan_form_instance_provider_mode import BarcodeScanFormInstanceProviderMode
from ..models.barcode_scan_form_instance_provider_type import BarcodeScanFormInstanceProviderType
from ..models.form_barcode_value_reference import FormBarcodeValueReference
from ..models.form_field_value_reference import FormFieldValueReference
from ..models.form_raw_string_query_value_part import FormRawStringQueryValuePart
from ..types import UNSET, Unset

T = TypeVar("T", bound="BarcodeScanFormInstanceProvider")


@attr.s(auto_attribs=True, repr=False)
class BarcodeScanFormInstanceProvider:
    """  """

    _identifier_type: Union[Unset, BarcodeScanFormInstanceProviderIdentifierType] = UNSET
    _mode: Union[Unset, BarcodeScanFormInstanceProviderMode] = UNSET
    _query_value_from: Union[
        Unset,
        List[
            Union[
                FormBarcodeValueReference, FormRawStringQueryValuePart, FormFieldValueReference, UnknownType
            ]
        ],
    ] = UNSET
    _type: Union[Unset, BarcodeScanFormInstanceProviderType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("identifier_type={}".format(repr(self._identifier_type)))
        fields.append("mode={}".format(repr(self._mode)))
        fields.append("query_value_from={}".format(repr(self._query_value_from)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BarcodeScanFormInstanceProvider({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        identifier_type: Union[Unset, int] = UNSET
        if not isinstance(self._identifier_type, Unset):
            identifier_type = self._identifier_type.value

        mode: Union[Unset, int] = UNSET
        if not isinstance(self._mode, Unset):
            mode = self._mode.value

        query_value_from: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._query_value_from, Unset):
            query_value_from = []
            for query_value_from_item_data in self._query_value_from:
                if isinstance(query_value_from_item_data, UnknownType):
                    query_value_from_item = query_value_from_item_data.value
                elif isinstance(query_value_from_item_data, FormBarcodeValueReference):
                    query_value_from_item = query_value_from_item_data.to_dict()

                elif isinstance(query_value_from_item_data, FormRawStringQueryValuePart):
                    query_value_from_item = query_value_from_item_data.to_dict()

                else:
                    query_value_from_item = query_value_from_item_data.to_dict()

                query_value_from.append(query_value_from_item)

        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if identifier_type is not UNSET:
            field_dict["identifierType"] = identifier_type
        if mode is not UNSET:
            field_dict["mode"] = mode
        if query_value_from is not UNSET:
            field_dict["queryValueFrom"] = query_value_from
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_identifier_type() -> Union[Unset, BarcodeScanFormInstanceProviderIdentifierType]:
            identifier_type = UNSET
            _identifier_type = d.pop("identifierType")
            if _identifier_type is not None and _identifier_type is not UNSET:
                try:
                    identifier_type = BarcodeScanFormInstanceProviderIdentifierType(_identifier_type)
                except ValueError:
                    identifier_type = BarcodeScanFormInstanceProviderIdentifierType.of_unknown(
                        _identifier_type
                    )

            return identifier_type

        try:
            identifier_type = get_identifier_type()
        except KeyError:
            if strict:
                raise
            identifier_type = cast(Union[Unset, BarcodeScanFormInstanceProviderIdentifierType], UNSET)

        def get_mode() -> Union[Unset, BarcodeScanFormInstanceProviderMode]:
            mode = UNSET
            _mode = d.pop("mode")
            if _mode is not None and _mode is not UNSET:
                try:
                    mode = BarcodeScanFormInstanceProviderMode(_mode)
                except ValueError:
                    mode = BarcodeScanFormInstanceProviderMode.of_unknown(_mode)

            return mode

        try:
            mode = get_mode()
        except KeyError:
            if strict:
                raise
            mode = cast(Union[Unset, BarcodeScanFormInstanceProviderMode], UNSET)

        def get_query_value_from() -> Union[
            Unset,
            List[
                Union[
                    FormBarcodeValueReference,
                    FormRawStringQueryValuePart,
                    FormFieldValueReference,
                    UnknownType,
                ]
            ],
        ]:
            query_value_from = []
            _query_value_from = d.pop("queryValueFrom")
            for query_value_from_item_data in _query_value_from or []:

                def _parse_query_value_from_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    FormBarcodeValueReference,
                    FormRawStringQueryValuePart,
                    FormFieldValueReference,
                    UnknownType,
                ]:
                    query_value_from_item: Union[
                        FormBarcodeValueReference,
                        FormRawStringQueryValuePart,
                        FormFieldValueReference,
                        UnknownType,
                    ]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        query_value_from_item = FormBarcodeValueReference.from_dict(data, strict=True)

                        return query_value_from_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        query_value_from_item = FormRawStringQueryValuePart.from_dict(data, strict=True)

                        return query_value_from_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        query_value_from_item = FormFieldValueReference.from_dict(data, strict=True)

                        return query_value_from_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                query_value_from_item = _parse_query_value_from_item(query_value_from_item_data)

                query_value_from.append(query_value_from_item)

            return query_value_from

        try:
            query_value_from = get_query_value_from()
        except KeyError:
            if strict:
                raise
            query_value_from = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            FormBarcodeValueReference,
                            FormRawStringQueryValuePart,
                            FormFieldValueReference,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_type() -> Union[Unset, BarcodeScanFormInstanceProviderType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = BarcodeScanFormInstanceProviderType(_type)
                except ValueError:
                    type = BarcodeScanFormInstanceProviderType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, BarcodeScanFormInstanceProviderType], UNSET)

        barcode_scan_form_instance_provider = cls(
            identifier_type=identifier_type,
            mode=mode,
            query_value_from=query_value_from,
            type=type,
        )

        barcode_scan_form_instance_provider.additional_properties = d
        return barcode_scan_form_instance_provider

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def identifier_type(self) -> BarcodeScanFormInstanceProviderIdentifierType:
        """How we identify the entity that we should query for. Because Benchling entities have several different ways of declaring identity, we also must allow for different ways they can be distinguished by their barcodes."""
        if isinstance(self._identifier_type, Unset):
            raise NotPresentError(self, "identifier_type")
        return self._identifier_type

    @identifier_type.setter
    def identifier_type(self, value: BarcodeScanFormInstanceProviderIdentifierType) -> None:
        self._identifier_type = value

    @identifier_type.deleter
    def identifier_type(self) -> None:
        self._identifier_type = UNSET

    @property
    def mode(self) -> BarcodeScanFormInstanceProviderMode:
        """How we differentiate between single scan mode and multiple scan mode. This will allow a user to either choose to scan and load one barcode at a time or multiple barcodes."""
        if isinstance(self._mode, Unset):
            raise NotPresentError(self, "mode")
        return self._mode

    @mode.setter
    def mode(self, value: BarcodeScanFormInstanceProviderMode) -> None:
        self._mode = value

    @mode.deleter
    def mode(self) -> None:
        self._mode = UNSET

    @property
    def query_value_from(
        self,
    ) -> List[
        Union[FormBarcodeValueReference, FormRawStringQueryValuePart, FormFieldValueReference, UnknownType]
    ]:
        """Because in some contexts barcodes are not necessarily unique, we must also allow for ways to construct the actual value that we should use to match a Benchling entity with. An example here would be entities whose names are constructed like: <barcode>-<crop>"""
        if isinstance(self._query_value_from, Unset):
            raise NotPresentError(self, "query_value_from")
        return self._query_value_from

    @query_value_from.setter
    def query_value_from(
        self,
        value: List[
            Union[
                FormBarcodeValueReference, FormRawStringQueryValuePart, FormFieldValueReference, UnknownType
            ]
        ],
    ) -> None:
        self._query_value_from = value

    @query_value_from.deleter
    def query_value_from(self) -> None:
        self._query_value_from = UNSET

    @property
    def type(self) -> BarcodeScanFormInstanceProviderType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: BarcodeScanFormInstanceProviderType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry_link import EntryLink
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTableCell")


@attr.s(auto_attribs=True, repr=False)
class EntryTableCell:
    """  """

    _link: Union[Unset, EntryLink] = UNSET
    _text: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("link={}".format(repr(self._link)))
        fields.append("text={}".format(repr(self._text)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntryTableCell({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        link: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._link, Unset):
            link = self._link.to_dict()

        text = self._text

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if link is not UNSET:
            field_dict["link"] = link
        if text is not UNSET:
            field_dict["text"] = text

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_link() -> Union[Unset, EntryLink]:
            link: Union[Unset, Union[Unset, EntryLink]] = UNSET
            _link = d.pop("link")

            if not isinstance(_link, Unset):
                link = EntryLink.from_dict(_link)

            return link

        try:
            link = get_link()
        except KeyError:
            if strict:
                raise
            link = cast(Union[Unset, EntryLink], UNSET)

        def get_text() -> Union[Unset, str]:
            text = d.pop("text")
            return text

        try:
            text = get_text()
        except KeyError:
            if strict:
                raise
            text = cast(Union[Unset, str], UNSET)

        entry_table_cell = cls(
            link=link,
            text=text,
        )

        entry_table_cell.additional_properties = d
        return entry_table_cell

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def link(self) -> EntryLink:
        """Links are contained within notes to reference resources that live outside of the entry. A link can target an external resource via an http(s):// hyperlink or a Benchling resource via @-mentions and drag-n-drop."""
        if isinstance(self._link, Unset):
            raise NotPresentError(self, "link")
        return self._link

    @link.setter
    def link(self, value: EntryLink) -> None:
        self._link = value

    @link.deleter
    def link(self) -> None:
        self._link = UNSET

    @property
    def text(self) -> str:
        """The textual content of the cell. If the cell was originally a formula, this will be the evaluated version of the formula."""
        if isinstance(self._text, Unset):
            raise NotPresentError(self, "text")
        return self._text

    @text.setter
    def text(self, value: str) -> None:
        self._text = value

    @text.deleter
    def text(self) -> None:
        self._text = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.async_task_errors import AsyncTaskErrors
from ..models.async_task_status import AsyncTaskStatus
from ..models.exports_async_task_response import ExportsAsyncTaskResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExportsAsyncTask")


@attr.s(auto_attribs=True, repr=False)
class ExportsAsyncTask:
    """  """

    _status: AsyncTaskStatus
    _response: Union[Unset, ExportsAsyncTaskResponse] = UNSET
    _errors: Union[Unset, AsyncTaskErrors] = UNSET
    _message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("status={}".format(repr(self._status)))
        fields.append("response={}".format(repr(self._response)))
        fields.append("errors={}".format(repr(self._errors)))
        fields.append("message={}".format(repr(self._message)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ExportsAsyncTask({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        status = self._status.value

        response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._response, Unset):
            response = self._response.to_dict()

        errors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._errors, Unset):
            errors = self._errors.to_dict()

        message = self._message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if status is not UNSET:
            field_dict["status"] = status
        if response is not UNSET:
            field_dict["response"] = response
        if errors is not UNSET:
            field_dict["errors"] = errors
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_status() -> AsyncTaskStatus:
            _status = d.pop("status")
            try:
                status = AsyncTaskStatus(_status)
            except ValueError:
                status = AsyncTaskStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(AsyncTaskStatus, UNSET)

        def get_response() -> Union[Unset, ExportsAsyncTaskResponse]:
            response: Union[Unset, Union[Unset, ExportsAsyncTaskResponse]] = UNSET
            _response = d.pop("response")

            if not isinstance(_response, Unset):
                response = ExportsAsyncTaskResponse.from_dict(_response)

            return response

        try:
            response = get_response()
        except KeyError:
            if strict:
                raise
            response = cast(Union[Unset, ExportsAsyncTaskResponse], UNSET)

        def get_errors() -> Union[Unset, AsyncTaskErrors]:
            errors: Union[Unset, Union[Unset, AsyncTaskErrors]] = UNSET
            _errors = d.pop("errors")

            if not isinstance(_errors, Unset):
                errors = AsyncTaskErrors.from_dict(_errors)

            return errors

        try:
            errors = get_errors()
        except KeyError:
            if strict:
                raise
            errors = cast(Union[Unset, AsyncTaskErrors], UNSET)

        def get_message() -> Union[Unset, str]:
            message = d.pop("message")
            return message

        try:
            message = get_message()
        except KeyError:
            if strict:
                raise
            message = cast(Union[Unset, str], UNSET)

        exports_async_task = cls(
            status=status,
            response=response,
            errors=errors,
            message=message,
        )

        exports_async_task.additional_properties = d
        return exports_async_task

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def status(self) -> AsyncTaskStatus:
        """ The current state of the task. """
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: AsyncTaskStatus) -> None:
        self._status = value

    @property
    def response(self) -> ExportsAsyncTaskResponse:
        if isinstance(self._response, Unset):
            raise NotPresentError(self, "response")
        return self._response

    @response.setter
    def response(self, value: ExportsAsyncTaskResponse) -> None:
        self._response = value

    @response.deleter
    def response(self) -> None:
        self._response = UNSET

    @property
    def errors(self) -> AsyncTaskErrors:
        """Present only when status is FAILED for a bulk task. Contains information about the individual errors in the bulk task."""
        if isinstance(self._errors, Unset):
            raise NotPresentError(self, "errors")
        return self._errors

    @errors.setter
    def errors(self, value: AsyncTaskErrors) -> None:
        self._errors = value

    @errors.deleter
    def errors(self) -> None:
        self._errors = UNSET

    @property
    def message(self) -> str:
        """ Present only when status is FAILED. Contains information about the error. """
        if isinstance(self._message, Unset):
            raise NotPresentError(self, "message")
        return self._message

    @message.setter
    def message(self, value: str) -> None:
        self._message = value

    @message.deleter
    def message(self) -> None:
        self._message = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class MixturePrepTableNotePartType(Enums.KnownString):
    MIXTURE_PREP_TABLE = "mixture_prep_table"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "MixturePrepTableNotePartType":
        if not isinstance(val, str):
            raise ValueError(f"Value of MixturePrepTableNotePartType must be a string (encountered: {val})")
        newcls = Enum("MixturePrepTableNotePartType", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(MixturePrepTableNotePartType, getattr(newcls, "_UNKNOWN"))



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...models.schema_fields_query_param import SchemaFieldsQueryParam
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    workflow_task_group_ids: Union[Unset, str] = UNSET,
    workflow_task_ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    creation_origin_ids: Union[Unset, str] = UNSET,
    linked_item_idsany_of: Union[Unset, str] = UNSET,
    linked_item_idsall_of: Union[Unset, str] = UNSET,
    linked_item_idsnone_of: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    source_task_idsany_of: Union[Unset, str] = UNSET,
    source_task_idsall_of: Union[Unset, str] = UNSET,
    source_output_idsany_of: Union[Unset, str] = UNSET,
    source_output_idsall_of: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/workflow-outputs".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_schema_fields: Union[Unset, Dict[str, Any]] = UNSET
    if not isinstance(schema_fields, Unset):
        json_schema_fields = schema_fields.to_dict()

    json_modified_at: Union[Unset, str] = UNSET
    if not isinstance(modified_at, Unset):
        json_modified_at = modified_at.isoformat()

    params: Dict[str, Any] = {}
    if not isinstance(ids, Unset) and ids is not None:
        params["ids"] = ids
    if not isinstance(workflow_task_group_ids, Unset) and workflow_task_group_ids is not None:
        params["workflowTaskGroupIds"] = workflow_task_group_ids
    if not isinstance(workflow_task_ids, Unset) and workflow_task_ids is not None:
        params["workflowTaskIds"] = workflow_task_ids
    if not isinstance(schema_id, Unset) and schema_id is not None:
        params["schemaId"] = schema_id
    if not isinstance(watcher_ids, Unset) and watcher_ids is not None:
        params["watcherIds"] = watcher_ids
    if not isinstance(responsible_team_ids, Unset) and responsible_team_ids is not None:
        params["responsibleTeamIds"] = responsible_team_ids
    if (
        not isinstance(flowchart_workflow_task_group_ids, Unset)
        and flowchart_workflow_task_group_ids is not None
    ):
        params["flowchartWorkflowTaskGroupIds"] = flowchart_workflow_task_group_ids
    if not isinstance(flowchart_task_schema_id, Unset) and flowchart_task_schema_id is not None:
        params["flowchartTaskSchemaId"] = flowchart_task_schema_id
    if not isinstance(flowchart_creator_ids, Unset) and flowchart_creator_ids is not None:
        params["flowchartCreatorIds"] = flowchart_creator_ids
    if not isinstance(flowchart_watcher_ids, Unset) and flowchart_watcher_ids is not None:
        params["flowchartWatcherIds"] = flowchart_watcher_ids
    if not isinstance(creation_origin_ids, Unset) and creation_origin_ids is not None:
        params["creationOriginIds"] = creation_origin_ids
    if not isinstance(linked_item_idsany_of, Unset) and linked_item_idsany_of is not None:
        params["linkedItemIds.anyOf"] = linked_item_idsany_of
    if not isinstance(linked_item_idsall_of, Unset) and linked_item_idsall_of is not None:
        params["linkedItemIds.allOf"] = linked_item_idsall_of
    if not isinstance(linked_item_idsnone_of, Unset) and linked_item_idsnone_of is not None:
        params["linkedItemIds.noneOf"] = linked_item_idsnone_of
    if not isinstance(json_schema_fields, Unset) and json_schema_fields is not None:
        params.update(json_schema_fields)
    if not isinstance(name, Unset) and name is not None:
        params["name"] = name
    if not isinstance(name_includes, Unset) and name_includes is not None:
        params["nameIncludes"] = name_includes
    if not isinstance(creator_ids, Unset) and creator_ids is not None:
        params["creatorIds"] = creator_ids
    if not isinstance(json_modified_at, Unset) and json_modified_at is not None:
        params["modifiedAt"] = json_modified_at
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(display_ids, Unset) and display_ids is not None:
        params["displayIds"] = display_ids
    if not isinstance(archive_reason, Unset) and archive_reason is not None:
        params["archiveReason"] = archive_reason
    if not isinstance(source_task_idsany_of, Unset) and source_task_idsany_of is not None:
        params["sourceTaskIds.anyOf"] = source_task_idsany_of
    if not isinstance(source_task_idsall_of, Unset) and source_task_idsall_of is not None:
        params["sourceTaskIds.allOf"] = source_task_idsall_of
    if not isinstance(source_output_idsany_of, Unset) and source_output_idsany_of is not None:
        params["sourceOutputIds.anyOf"] = source_output_idsany_of
    if not isinstance(source_output_idsall_of, Unset) and source_output_idsall_of is not None:
        params["sourceOutputIds.allOf"] = source_output_idsall_of

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[BadRequestError]:
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[BadRequestError]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    workflow_task_group_ids: Union[Unset, str] = UNSET,
    workflow_task_ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    creation_origin_ids: Union[Unset, str] = UNSET,
    linked_item_idsany_of: Union[Unset, str] = UNSET,
    linked_item_idsall_of: Union[Unset, str] = UNSET,
    linked_item_idsnone_of: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    source_task_idsany_of: Union[Unset, str] = UNSET,
    source_task_idsall_of: Union[Unset, str] = UNSET,
    source_output_idsany_of: Union[Unset, str] = UNSET,
    source_output_idsall_of: Union[Unset, str] = UNSET,
) -> Response[BadRequestError]:
    kwargs = _get_kwargs(
        client=client,
        ids=ids,
        workflow_task_group_ids=workflow_task_group_ids,
        workflow_task_ids=workflow_task_ids,
        schema_id=schema_id,
        watcher_ids=watcher_ids,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        creation_origin_ids=creation_origin_ids,
        linked_item_idsany_of=linked_item_idsany_of,
        linked_item_idsall_of=linked_item_idsall_of,
        linked_item_idsnone_of=linked_item_idsnone_of,
        schema_fields=schema_fields,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
        source_task_idsany_of=source_task_idsany_of,
        source_task_idsall_of=source_task_idsall_of,
        source_output_idsany_of=source_output_idsany_of,
        source_output_idsall_of=source_output_idsall_of,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    workflow_task_group_ids: Union[Unset, str] = UNSET,
    workflow_task_ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    creation_origin_ids: Union[Unset, str] = UNSET,
    linked_item_idsany_of: Union[Unset, str] = UNSET,
    linked_item_idsall_of: Union[Unset, str] = UNSET,
    linked_item_idsnone_of: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    source_task_idsany_of: Union[Unset, str] = UNSET,
    source_task_idsall_of: Union[Unset, str] = UNSET,
    source_output_idsany_of: Union[Unset, str] = UNSET,
    source_output_idsall_of: Union[Unset, str] = UNSET,
) -> Optional[BadRequestError]:
    """ List workflow outputs """

    return sync_detailed(
        client=client,
        ids=ids,
        workflow_task_group_ids=workflow_task_group_ids,
        workflow_task_ids=workflow_task_ids,
        schema_id=schema_id,
        watcher_ids=watcher_ids,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        creation_origin_ids=creation_origin_ids,
        linked_item_idsany_of=linked_item_idsany_of,
        linked_item_idsall_of=linked_item_idsall_of,
        linked_item_idsnone_of=linked_item_idsnone_of,
        schema_fields=schema_fields,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
        source_task_idsany_of=source_task_idsany_of,
        source_task_idsall_of=source_task_idsall_of,
        source_output_idsany_of=source_output_idsany_of,
        source_output_idsall_of=source_output_idsall_of,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    workflow_task_group_ids: Union[Unset, str] = UNSET,
    workflow_task_ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    creation_origin_ids: Union[Unset, str] = UNSET,
    linked_item_idsany_of: Union[Unset, str] = UNSET,
    linked_item_idsall_of: Union[Unset, str] = UNSET,
    linked_item_idsnone_of: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    source_task_idsany_of: Union[Unset, str] = UNSET,
    source_task_idsall_of: Union[Unset, str] = UNSET,
    source_output_idsany_of: Union[Unset, str] = UNSET,
    source_output_idsall_of: Union[Unset, str] = UNSET,
) -> Response[BadRequestError]:
    kwargs = _get_kwargs(
        client=client,
        ids=ids,
        workflow_task_group_ids=workflow_task_group_ids,
        workflow_task_ids=workflow_task_ids,
        schema_id=schema_id,
        watcher_ids=watcher_ids,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        creation_origin_ids=creation_origin_ids,
        linked_item_idsany_of=linked_item_idsany_of,
        linked_item_idsall_of=linked_item_idsall_of,
        linked_item_idsnone_of=linked_item_idsnone_of,
        schema_fields=schema_fields,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
        source_task_idsany_of=source_task_idsany_of,
        source_task_idsall_of=source_task_idsall_of,
        source_output_idsany_of=source_output_idsany_of,
        source_output_idsall_of=source_output_idsall_of,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    workflow_task_group_ids: Union[Unset, str] = UNSET,
    workflow_task_ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    creation_origin_ids: Union[Unset, str] = UNSET,
    linked_item_idsany_of: Union[Unset, str] = UNSET,
    linked_item_idsall_of: Union[Unset, str] = UNSET,
    linked_item_idsnone_of: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    source_task_idsany_of: Union[Unset, str] = UNSET,
    source_task_idsall_of: Union[Unset, str] = UNSET,
    source_output_idsany_of: Union[Unset, str] = UNSET,
    source_output_idsall_of: Union[Unset, str] = UNSET,
) -> Optional[BadRequestError]:
    """ List workflow outputs """

    return (
        await asyncio_detailed(
            client=client,
            ids=ids,
            workflow_task_group_ids=workflow_task_group_ids,
            workflow_task_ids=workflow_task_ids,
            schema_id=schema_id,
            watcher_ids=watcher_ids,
            responsible_team_ids=responsible_team_ids,
            flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
            flowchart_task_schema_id=flowchart_task_schema_id,
            flowchart_creator_ids=flowchart_creator_ids,
            flowchart_watcher_ids=flowchart_watcher_ids,
            creation_origin_ids=creation_origin_ids,
            linked_item_idsany_of=linked_item_idsany_of,
            linked_item_idsall_of=linked_item_idsall_of,
            linked_item_idsnone_of=linked_item_idsnone_of,
            schema_fields=schema_fields,
            name=name,
            name_includes=name_includes,
            creator_ids=creator_ids,
            modified_at=modified_at,
            next_token=next_token,
            page_size=page_size,
            display_ids=display_ids,
            archive_reason=archive_reason,
            source_task_idsany_of=source_task_idsany_of,
            source_task_idsall_of=source_task_idsall_of,
            source_output_idsany_of=source_output_idsany_of,
            source_output_idsall_of=source_output_idsall_of,
        )
    ).parsed



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class BenchlingAppManifestManifestVersion(Enums.KnownInt):
    VALUE_1 = 1

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: int) -> "BenchlingAppManifestManifestVersion":
        if not isinstance(val, int):
            raise ValueError(
                f"Value of BenchlingAppManifestManifestVersion must be an int (encountered: {val})"
            )
        newcls = Enum("BenchlingAppManifestManifestVersion", {"_UNKNOWN": val}, type=Enums.UnknownInt)  # type: ignore
        return cast(BenchlingAppManifestManifestVersion, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.mixture_bulk_update import MixtureBulkUpdate
from ..types import UNSET, Unset

T = TypeVar("T", bound="MixturesBulkUpdateRequest")


@attr.s(auto_attribs=True, repr=False)
class MixturesBulkUpdateRequest:
    """  """

    _mixtures: Union[Unset, List[MixtureBulkUpdate]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("mixtures={}".format(repr(self._mixtures)))
        return "MixturesBulkUpdateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        mixtures: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._mixtures, Unset):
            mixtures = []
            for mixtures_item_data in self._mixtures:
                mixtures_item = mixtures_item_data.to_dict()

                mixtures.append(mixtures_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if mixtures is not UNSET:
            field_dict["mixtures"] = mixtures

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_mixtures() -> Union[Unset, List[MixtureBulkUpdate]]:
            mixtures = []
            _mixtures = d.pop("mixtures")
            for mixtures_item_data in _mixtures or []:
                mixtures_item = MixtureBulkUpdate.from_dict(mixtures_item_data, strict=False)

                mixtures.append(mixtures_item)

            return mixtures

        try:
            mixtures = get_mixtures()
        except KeyError:
            if strict:
                raise
            mixtures = cast(Union[Unset, List[MixtureBulkUpdate]], UNSET)

        mixtures_bulk_update_request = cls(
            mixtures=mixtures,
        )

        return mixtures_bulk_update_request

    @property
    def mixtures(self) -> List[MixtureBulkUpdate]:
        if isinstance(self._mixtures, Unset):
            raise NotPresentError(self, "mixtures")
        return self._mixtures

    @mixtures.setter
    def mixtures(self, value: List[MixtureBulkUpdate]) -> None:
        self._mixtures = value

    @mixtures.deleter
    def mixtures(self) -> None:
        self._mixtures = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry_table_cell import EntryTableCell
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTableRow")


@attr.s(auto_attribs=True, repr=False)
class EntryTableRow:
    """ Each has property 'cells' that is an array of cell objects """

    _cells: Union[Unset, List[EntryTableCell]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("cells={}".format(repr(self._cells)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntryTableRow({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        cells: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._cells, Unset):
            cells = []
            for cells_item_data in self._cells:
                cells_item = cells_item_data.to_dict()

                cells.append(cells_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if cells is not UNSET:
            field_dict["cells"] = cells

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_cells() -> Union[Unset, List[EntryTableCell]]:
            cells = []
            _cells = d.pop("cells")
            for cells_item_data in _cells or []:
                cells_item = EntryTableCell.from_dict(cells_item_data, strict=False)

                cells.append(cells_item)

            return cells

        try:
            cells = get_cells()
        except KeyError:
            if strict:
                raise
            cells = cast(Union[Unset, List[EntryTableCell]], UNSET)

        entry_table_row = cls(
            cells=cells,
        )

        entry_table_row.additional_properties = d
        return entry_table_row

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def cells(self) -> List[EntryTableCell]:
        if isinstance(self._cells, Unset):
            raise NotPresentError(self, "cells")
        return self._cells

    @cells.setter
    def cells(self, value: List[EntryTableCell]) -> None:
        self._cells = value

    @cells.deleter
    def cells(self) -> None:
        self._cells = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.lab_automation_benchling_app_error import LabAutomationBenchlingAppError
from ..types import UNSET, Unset

T = TypeVar("T", bound="LabAutomationTransformUpdate")


@attr.s(auto_attribs=True, repr=False)
class LabAutomationTransformUpdate:
    """  """

    _blob_id: Union[Unset, str] = UNSET
    _errors: Union[Unset, List[LabAutomationBenchlingAppError]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("blob_id={}".format(repr(self._blob_id)))
        fields.append("errors={}".format(repr(self._errors)))
        return "LabAutomationTransformUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        blob_id = self._blob_id
        errors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._errors, Unset):
            errors = []
            for errors_item_data in self._errors:
                errors_item = errors_item_data.to_dict()

                errors.append(errors_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if blob_id is not UNSET:
            field_dict["blobId"] = blob_id
        if errors is not UNSET:
            field_dict["errors"] = errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_blob_id() -> Union[Unset, str]:
            blob_id = d.pop("blobId")
            return blob_id

        try:
            blob_id = get_blob_id()
        except KeyError:
            if strict:
                raise
            blob_id = cast(Union[Unset, str], UNSET)

        def get_errors() -> Union[Unset, List[LabAutomationBenchlingAppError]]:
            errors = []
            _errors = d.pop("errors")
            for errors_item_data in _errors or []:
                errors_item = LabAutomationBenchlingAppError.from_dict(errors_item_data, strict=False)

                errors.append(errors_item)

            return errors

        try:
            errors = get_errors()
        except KeyError:
            if strict:
                raise
            errors = cast(Union[Unset, List[LabAutomationBenchlingAppError]], UNSET)

        lab_automation_transform_update = cls(
            blob_id=blob_id,
            errors=errors,
        )

        return lab_automation_transform_update

    @property
    def blob_id(self) -> str:
        if isinstance(self._blob_id, Unset):
            raise NotPresentError(self, "blob_id")
        return self._blob_id

    @blob_id.setter
    def blob_id(self, value: str) -> None:
        self._blob_id = value

    @blob_id.deleter
    def blob_id(self) -> None:
        self._blob_id = UNSET

    @property
    def errors(self) -> List[LabAutomationBenchlingAppError]:
        if isinstance(self._errors, Unset):
            raise NotPresentError(self, "errors")
        return self._errors

    @errors.setter
    def errors(self, value: List[LabAutomationBenchlingAppError]) -> None:
        self._errors = value

    @errors.deleter
    def errors(self) -> None:
        self._errors = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.aa_sequence_with_entity_type import AaSequenceWithEntityType
from ..models.custom_entity_with_entity_type import CustomEntityWithEntityType
from ..models.dna_oligo_with_entity_type import DnaOligoWithEntityType
from ..models.dna_sequence_with_entity_type import DnaSequenceWithEntityType
from ..models.mixture_with_entity_type import MixtureWithEntityType
from ..models.rna_oligo_with_entity_type import RnaOligoWithEntityType
from ..types import UNSET, Unset

T = TypeVar("T", bound="RegisteredEntitiesList")


@attr.s(auto_attribs=True, repr=False)
class RegisteredEntitiesList:
    """  """

    _entities: Union[
        Unset,
        List[
            Union[
                DnaSequenceWithEntityType,
                CustomEntityWithEntityType,
                AaSequenceWithEntityType,
                MixtureWithEntityType,
                DnaOligoWithEntityType,
                RnaOligoWithEntityType,
                UnknownType,
            ]
        ],
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entities={}".format(repr(self._entities)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RegisteredEntitiesList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entities: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._entities, Unset):
            entities = []
            for entities_item_data in self._entities:
                if isinstance(entities_item_data, UnknownType):
                    entities_item = entities_item_data.value
                elif isinstance(entities_item_data, DnaSequenceWithEntityType):
                    entities_item = entities_item_data.to_dict()

                elif isinstance(entities_item_data, CustomEntityWithEntityType):
                    entities_item = entities_item_data.to_dict()

                elif isinstance(entities_item_data, AaSequenceWithEntityType):
                    entities_item = entities_item_data.to_dict()

                elif isinstance(entities_item_data, MixtureWithEntityType):
                    entities_item = entities_item_data.to_dict()

                elif isinstance(entities_item_data, DnaOligoWithEntityType):
                    entities_item = entities_item_data.to_dict()

                else:
                    entities_item = entities_item_data.to_dict()

                entities.append(entities_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entities is not UNSET:
            field_dict["entities"] = entities

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entities() -> Union[
            Unset,
            List[
                Union[
                    DnaSequenceWithEntityType,
                    CustomEntityWithEntityType,
                    AaSequenceWithEntityType,
                    MixtureWithEntityType,
                    DnaOligoWithEntityType,
                    RnaOligoWithEntityType,
                    UnknownType,
                ]
            ],
        ]:
            entities = []
            _entities = d.pop("entities")
            for entities_item_data in _entities or []:

                def _parse_entities_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    DnaSequenceWithEntityType,
                    CustomEntityWithEntityType,
                    AaSequenceWithEntityType,
                    MixtureWithEntityType,
                    DnaOligoWithEntityType,
                    RnaOligoWithEntityType,
                    UnknownType,
                ]:
                    entities_item: Union[
                        DnaSequenceWithEntityType,
                        CustomEntityWithEntityType,
                        AaSequenceWithEntityType,
                        MixtureWithEntityType,
                        DnaOligoWithEntityType,
                        RnaOligoWithEntityType,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("entityType"))
                    if discriminator_value is not None:
                        if discriminator_value == "aa_sequence":
                            entities_item = AaSequenceWithEntityType.from_dict(data, strict=False)

                            return entities_item
                        if discriminator_value == "custom_entity":
                            entities_item = CustomEntityWithEntityType.from_dict(data, strict=False)

                            return entities_item
                        if discriminator_value == "dna_oligo":
                            entities_item = DnaOligoWithEntityType.from_dict(data, strict=False)

                            return entities_item
                        if discriminator_value == "dna_sequence":
                            entities_item = DnaSequenceWithEntityType.from_dict(data, strict=False)

                            return entities_item
                        if discriminator_value == "mixture":
                            entities_item = MixtureWithEntityType.from_dict(data, strict=False)

                            return entities_item
                        if discriminator_value == "rna_oligo":
                            entities_item = RnaOligoWithEntityType.from_dict(data, strict=False)

                            return entities_item

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = DnaSequenceWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = CustomEntityWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = AaSequenceWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = MixtureWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = DnaOligoWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        entities_item = RnaOligoWithEntityType.from_dict(data, strict=True)

                        return entities_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                entities_item = _parse_entities_item(entities_item_data)

                entities.append(entities_item)

            return entities

        try:
            entities = get_entities()
        except KeyError:
            if strict:
                raise
            entities = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            DnaSequenceWithEntityType,
                            CustomEntityWithEntityType,
                            AaSequenceWithEntityType,
                            MixtureWithEntityType,
                            DnaOligoWithEntityType,
                            RnaOligoWithEntityType,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        registered_entities_list = cls(
            entities=entities,
        )

        registered_entities_list.additional_properties = d
        return registered_entities_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entities(
        self,
    ) -> List[
        Union[
            DnaSequenceWithEntityType,
            CustomEntityWithEntityType,
            AaSequenceWithEntityType,
            MixtureWithEntityType,
            DnaOligoWithEntityType,
            RnaOligoWithEntityType,
            UnknownType,
        ]
    ]:
        if isinstance(self._entities, Unset):
            raise NotPresentError(self, "entities")
        return self._entities

    @entities.setter
    def entities(
        self,
        value: List[
            Union[
                DnaSequenceWithEntityType,
                CustomEntityWithEntityType,
                AaSequenceWithEntityType,
                MixtureWithEntityType,
                DnaOligoWithEntityType,
                RnaOligoWithEntityType,
                UnknownType,
            ]
        ],
    ) -> None:
        self._entities = value

    @entities.deleter
    def entities(self) -> None:
        self._entities = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class TranslationGeneticCode(Enums.KnownString):
    STANDARD = "STANDARD"
    VERTEBRATE_MITOCHONDRIAL = "VERTEBRATE_MITOCHONDRIAL"
    YEAST_MITOCHONDRIAL = "YEAST_MITOCHONDRIAL"
    MOLD_PROTOZOAN_COELENTERATE_MITOCHONDRIAL_MYCOPLASMA_SPIROPLASMA = (
        "MOLD_PROTOZOAN_COELENTERATE_MITOCHONDRIAL_MYCOPLASMA_SPIROPLASMA"
    )
    INVERTEBRATE_MITOCHONDRIAL = "INVERTEBRATE_MITOCHONDRIAL"
    CILIATE_DASYCLADACEAN_HEXAMITA_NUCLEAR = "CILIATE_DASYCLADACEAN_HEXAMITA_NUCLEAR"
    ECHINODERM_FLATWORM_MITOCHONDRIAL = "ECHINODERM_FLATWORM_MITOCHONDRIAL"
    EUPLOTID_NUCLEAR = "EUPLOTID_NUCLEAR"
    BACTERIAL_ARCHAEAL_PLANT_PLASTID = "BACTERIAL_ARCHAEAL_PLANT_PLASTID"
    ALTERNATIVE_YEAST_NUCLEAR = "ALTERNATIVE_YEAST_NUCLEAR"
    ASCIDIAN_MITOCHONDRIAL = "ASCIDIAN_MITOCHONDRIAL"
    ALTERNATIVE_FLATWORM_MITOCHONDRIAL = "ALTERNATIVE_FLATWORM_MITOCHONDRIAL"
    CHLOROPHYCEAN_MITOCHONDRIAL = "CHLOROPHYCEAN_MITOCHONDRIAL"
    TREMATODE_MITOCHONDRIAL = "TREMATODE_MITOCHONDRIAL"
    SCENEDESMUS_OBLIQUUS_MITOCHONDRIAL = "SCENEDESMUS_OBLIQUUS_MITOCHONDRIAL"
    THRAUSTOCHYTRIUM_MITOCHONDRIAL = "THRAUSTOCHYTRIUM_MITOCHONDRIAL"
    RHABDOPLEURIDAE_MITOCHONDRIAL = "RHABDOPLEURIDAE_MITOCHONDRIAL"
    CANDIDATE_DIVISION_S_R1_GRACILIBACTERIA = "CANDIDATE_DIVISION_SR1_GRACILIBACTERIA"
    PACHYSOLEN_TANNOPHILUS_NUCLEAR = "PACHYSOLEN_TANNOPHILUS_NUCLEAR"
    MESODINIUM_NUCLEAR = "MESODINIUM_NUCLEAR"
    PERITRICH_NUCLEAR = "PERITRICH_NUCLEAR"
    CEPHALODISCIDAE_MITOCHONDRIAL_UAA_TYR = "CEPHALODISCIDAE_MITOCHONDRIAL_UAA_TYR"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "TranslationGeneticCode":
        if not isinstance(val, str):
            raise ValueError(f"Value of TranslationGeneticCode must be a string (encountered: {val})")
        newcls = Enum("TranslationGeneticCode", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(TranslationGeneticCode, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.creation_origin import CreationOrigin
from ..models.folder import Folder
from ..models.team_summary import TeamSummary
from ..models.user_summary import UserSummary
from ..models.workflow_output_summary import WorkflowOutputSummary
from ..models.workflow_task_group_execution_type import WorkflowTaskGroupExecutionType
from ..models.workflow_task_schema_summary import WorkflowTaskSchemaSummary
from ..models.workflow_task_summary import WorkflowTaskSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskGroup")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskGroup:
    """  """

    _execution_type: Union[Unset, WorkflowTaskGroupExecutionType] = UNSET
    _creation_origin: Union[Unset, CreationOrigin] = UNSET
    _creator: Union[Unset, UserSummary] = UNSET
    _folder: Union[Unset, Folder] = UNSET
    _outputs: Union[Unset, List[WorkflowOutputSummary]] = UNSET
    _responsible_team: Union[Unset, None, TeamSummary] = UNSET
    _watchers: Union[Unset, List[UserSummary]] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _workflow_task_schema: Union[Unset, WorkflowTaskSchemaSummary] = UNSET
    _workflow_tasks: Union[Unset, List[WorkflowTaskSummary]] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("execution_type={}".format(repr(self._execution_type)))
        fields.append("creation_origin={}".format(repr(self._creation_origin)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("folder={}".format(repr(self._folder)))
        fields.append("outputs={}".format(repr(self._outputs)))
        fields.append("responsible_team={}".format(repr(self._responsible_team)))
        fields.append("watchers={}".format(repr(self._watchers)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("workflow_task_schema={}".format(repr(self._workflow_task_schema)))
        fields.append("workflow_tasks={}".format(repr(self._workflow_tasks)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskGroup({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        execution_type: Union[Unset, int] = UNSET
        if not isinstance(self._execution_type, Unset):
            execution_type = self._execution_type.value

        creation_origin: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creation_origin, Unset):
            creation_origin = self._creation_origin.to_dict()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        folder: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._folder, Unset):
            folder = self._folder.to_dict()

        outputs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._outputs, Unset):
            outputs = []
            for outputs_item_data in self._outputs:
                outputs_item = outputs_item_data.to_dict()

                outputs.append(outputs_item)

        responsible_team: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._responsible_team, Unset):
            responsible_team = self._responsible_team.to_dict() if self._responsible_team else None

        watchers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._watchers, Unset):
            watchers = []
            for watchers_item_data in self._watchers:
                watchers_item = watchers_item_data.to_dict()

                watchers.append(watchers_item)

        web_url = self._web_url
        workflow_task_schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._workflow_task_schema, Unset):
            workflow_task_schema = self._workflow_task_schema.to_dict()

        workflow_tasks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._workflow_tasks, Unset):
            workflow_tasks = []
            for workflow_tasks_item_data in self._workflow_tasks:
                workflow_tasks_item = workflow_tasks_item_data.to_dict()

                workflow_tasks.append(workflow_tasks_item)

        display_id = self._display_id
        id = self._id
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if execution_type is not UNSET:
            field_dict["executionType"] = execution_type
        if creation_origin is not UNSET:
            field_dict["creationOrigin"] = creation_origin
        if creator is not UNSET:
            field_dict["creator"] = creator
        if folder is not UNSET:
            field_dict["folder"] = folder
        if outputs is not UNSET:
            field_dict["outputs"] = outputs
        if responsible_team is not UNSET:
            field_dict["responsibleTeam"] = responsible_team
        if watchers is not UNSET:
            field_dict["watchers"] = watchers
        if web_url is not UNSET:
            field_dict["webURL"] = web_url
        if workflow_task_schema is not UNSET:
            field_dict["workflowTaskSchema"] = workflow_task_schema
        if workflow_tasks is not UNSET:
            field_dict["workflowTasks"] = workflow_tasks
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_execution_type() -> Union[Unset, WorkflowTaskGroupExecutionType]:
            execution_type = UNSET
            _execution_type = d.pop("executionType")
            if _execution_type is not None and _execution_type is not UNSET:
                try:
                    execution_type = WorkflowTaskGroupExecutionType(_execution_type)
                except ValueError:
                    execution_type = WorkflowTaskGroupExecutionType.of_unknown(_execution_type)

            return execution_type

        try:
            execution_type = get_execution_type()
        except KeyError:
            if strict:
                raise
            execution_type = cast(Union[Unset, WorkflowTaskGroupExecutionType], UNSET)

        def get_creation_origin() -> Union[Unset, CreationOrigin]:
            creation_origin: Union[Unset, Union[Unset, CreationOrigin]] = UNSET
            _creation_origin = d.pop("creationOrigin")

            if not isinstance(_creation_origin, Unset):
                creation_origin = CreationOrigin.from_dict(_creation_origin)

            return creation_origin

        try:
            creation_origin = get_creation_origin()
        except KeyError:
            if strict:
                raise
            creation_origin = cast(Union[Unset, CreationOrigin], UNSET)

        def get_creator() -> Union[Unset, UserSummary]:
            creator: Union[Unset, Union[Unset, UserSummary]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = UserSummary.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, UserSummary], UNSET)

        def get_folder() -> Union[Unset, Folder]:
            folder: Union[Unset, Union[Unset, Folder]] = UNSET
            _folder = d.pop("folder")

            if not isinstance(_folder, Unset):
                folder = Folder.from_dict(_folder)

            return folder

        try:
            folder = get_folder()
        except KeyError:
            if strict:
                raise
            folder = cast(Union[Unset, Folder], UNSET)

        def get_outputs() -> Union[Unset, List[WorkflowOutputSummary]]:
            outputs = []
            _outputs = d.pop("outputs")
            for outputs_item_data in _outputs or []:
                outputs_item = WorkflowOutputSummary.from_dict(outputs_item_data, strict=False)

                outputs.append(outputs_item)

            return outputs

        try:
            outputs = get_outputs()
        except KeyError:
            if strict:
                raise
            outputs = cast(Union[Unset, List[WorkflowOutputSummary]], UNSET)

        def get_responsible_team() -> Union[Unset, None, TeamSummary]:
            responsible_team = None
            _responsible_team = d.pop("responsibleTeam")

            if _responsible_team is not None and not isinstance(_responsible_team, Unset):
                responsible_team = TeamSummary.from_dict(_responsible_team)

            return responsible_team

        try:
            responsible_team = get_responsible_team()
        except KeyError:
            if strict:
                raise
            responsible_team = cast(Union[Unset, None, TeamSummary], UNSET)

        def get_watchers() -> Union[Unset, List[UserSummary]]:
            watchers = []
            _watchers = d.pop("watchers")
            for watchers_item_data in _watchers or []:
                watchers_item = UserSummary.from_dict(watchers_item_data, strict=False)

                watchers.append(watchers_item)

            return watchers

        try:
            watchers = get_watchers()
        except KeyError:
            if strict:
                raise
            watchers = cast(Union[Unset, List[UserSummary]], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_workflow_task_schema() -> Union[Unset, WorkflowTaskSchemaSummary]:
            workflow_task_schema: Union[Unset, Union[Unset, WorkflowTaskSchemaSummary]] = UNSET
            _workflow_task_schema = d.pop("workflowTaskSchema")

            if not isinstance(_workflow_task_schema, Unset):
                workflow_task_schema = WorkflowTaskSchemaSummary.from_dict(_workflow_task_schema)

            return workflow_task_schema

        try:
            workflow_task_schema = get_workflow_task_schema()
        except KeyError:
            if strict:
                raise
            workflow_task_schema = cast(Union[Unset, WorkflowTaskSchemaSummary], UNSET)

        def get_workflow_tasks() -> Union[Unset, List[WorkflowTaskSummary]]:
            workflow_tasks = []
            _workflow_tasks = d.pop("workflowTasks")
            for workflow_tasks_item_data in _workflow_tasks or []:
                workflow_tasks_item = WorkflowTaskSummary.from_dict(workflow_tasks_item_data, strict=False)

                workflow_tasks.append(workflow_tasks_item)

            return workflow_tasks

        try:
            workflow_tasks = get_workflow_tasks()
        except KeyError:
            if strict:
                raise
            workflow_tasks = cast(Union[Unset, List[WorkflowTaskSummary]], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        workflow_task_group = cls(
            execution_type=execution_type,
            creation_origin=creation_origin,
            creator=creator,
            folder=folder,
            outputs=outputs,
            responsible_team=responsible_team,
            watchers=watchers,
            web_url=web_url,
            workflow_task_schema=workflow_task_schema,
            workflow_tasks=workflow_tasks,
            display_id=display_id,
            id=id,
            name=name,
        )

        workflow_task_group.additional_properties = d
        return workflow_task_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def execution_type(self) -> WorkflowTaskGroupExecutionType:
        """ The method by which the workflow is executed """
        if isinstance(self._execution_type, Unset):
            raise NotPresentError(self, "execution_type")
        return self._execution_type

    @execution_type.setter
    def execution_type(self, value: WorkflowTaskGroupExecutionType) -> None:
        self._execution_type = value

    @execution_type.deleter
    def execution_type(self) -> None:
        self._execution_type = UNSET

    @property
    def creation_origin(self) -> CreationOrigin:
        if isinstance(self._creation_origin, Unset):
            raise NotPresentError(self, "creation_origin")
        return self._creation_origin

    @creation_origin.setter
    def creation_origin(self, value: CreationOrigin) -> None:
        self._creation_origin = value

    @creation_origin.deleter
    def creation_origin(self) -> None:
        self._creation_origin = UNSET

    @property
    def creator(self) -> UserSummary:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: UserSummary) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def folder(self) -> Folder:
        if isinstance(self._folder, Unset):
            raise NotPresentError(self, "folder")
        return self._folder

    @folder.setter
    def folder(self, value: Folder) -> None:
        self._folder = value

    @folder.deleter
    def folder(self) -> None:
        self._folder = UNSET

    @property
    def outputs(self) -> List[WorkflowOutputSummary]:
        """ The outputs of the workflow task group """
        if isinstance(self._outputs, Unset):
            raise NotPresentError(self, "outputs")
        return self._outputs

    @outputs.setter
    def outputs(self, value: List[WorkflowOutputSummary]) -> None:
        self._outputs = value

    @outputs.deleter
    def outputs(self) -> None:
        self._outputs = UNSET

    @property
    def responsible_team(self) -> Optional[TeamSummary]:
        if isinstance(self._responsible_team, Unset):
            raise NotPresentError(self, "responsible_team")
        return self._responsible_team

    @responsible_team.setter
    def responsible_team(self, value: Optional[TeamSummary]) -> None:
        self._responsible_team = value

    @responsible_team.deleter
    def responsible_team(self) -> None:
        self._responsible_team = UNSET

    @property
    def watchers(self) -> List[UserSummary]:
        """ The users watching the workflow task group """
        if isinstance(self._watchers, Unset):
            raise NotPresentError(self, "watchers")
        return self._watchers

    @watchers.setter
    def watchers(self, value: List[UserSummary]) -> None:
        self._watchers = value

    @watchers.deleter
    def watchers(self) -> None:
        self._watchers = UNSET

    @property
    def web_url(self) -> str:
        """ URL of the workflow task group """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def workflow_task_schema(self) -> WorkflowTaskSchemaSummary:
        if isinstance(self._workflow_task_schema, Unset):
            raise NotPresentError(self, "workflow_task_schema")
        return self._workflow_task_schema

    @workflow_task_schema.setter
    def workflow_task_schema(self, value: WorkflowTaskSchemaSummary) -> None:
        self._workflow_task_schema = value

    @workflow_task_schema.deleter
    def workflow_task_schema(self) -> None:
        self._workflow_task_schema = UNSET

    @property
    def workflow_tasks(self) -> List[WorkflowTaskSummary]:
        """ The input tasks to the workflow task group """
        if isinstance(self._workflow_tasks, Unset):
            raise NotPresentError(self, "workflow_tasks")
        return self._workflow_tasks

    @workflow_tasks.setter
    def workflow_tasks(self, value: List[WorkflowTaskSummary]) -> None:
        self._workflow_tasks = value

    @workflow_tasks.deleter
    def workflow_tasks(self) -> None:
        self._workflow_tasks = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the workflow task group """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def id(self) -> str:
        """ The ID of the workflow task group """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        """ The name of the workflow task group """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.app_config_item_json_update_type import AppConfigItemJsonUpdateType
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppConfigItemJsonBulkUpdate")


@attr.s(auto_attribs=True, repr=False)
class AppConfigItemJsonBulkUpdate:
    """  """

    _id: str
    _type: AppConfigItemJsonUpdateType
    _value: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("value={}".format(repr(self._value)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AppConfigItemJsonBulkUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        type = self._type.value

        value = self._value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> str:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(str, UNSET)

        def get_type() -> AppConfigItemJsonUpdateType:
            _type = d.pop("type")
            try:
                type = AppConfigItemJsonUpdateType(_type)
            except ValueError:
                type = AppConfigItemJsonUpdateType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(AppConfigItemJsonUpdateType, UNSET)

        def get_value() -> str:
            value = d.pop("value")
            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(str, UNSET)

        app_config_item_json_bulk_update = cls(
            id=id,
            type=type,
            value=value,
        )

        app_config_item_json_bulk_update.additional_properties = d
        return app_config_item_json_bulk_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @property
    def type(self) -> AppConfigItemJsonUpdateType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: AppConfigItemJsonUpdateType) -> None:
        self._type = value

    @property
    def value(self) -> str:
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: str) -> None:
        self._value = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.registration_table_note_part_type import RegistrationTableNotePartType
from ..models.structured_table_column_info import StructuredTableColumnInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="RegistrationTableNotePart")


@attr.s(auto_attribs=True, repr=False)
class RegistrationTableNotePart:
    """  """

    _entity_schema_id: Union[Unset, str] = UNSET
    _type: Union[Unset, RegistrationTableNotePartType] = UNSET
    _api_id: Union[Unset, str] = UNSET
    _columns: Union[Unset, List[StructuredTableColumnInfo]] = UNSET
    _indentation: Union[Unset, int] = 0
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entity_schema_id={}".format(repr(self._entity_schema_id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("api_id={}".format(repr(self._api_id)))
        fields.append("columns={}".format(repr(self._columns)))
        fields.append("indentation={}".format(repr(self._indentation)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RegistrationTableNotePart({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entity_schema_id = self._entity_schema_id
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        api_id = self._api_id
        columns: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._columns, Unset):
            columns = []
            for columns_item_data in self._columns:
                columns_item = columns_item_data.to_dict()

                columns.append(columns_item)

        indentation = self._indentation

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entity_schema_id is not UNSET:
            field_dict["entitySchemaId"] = entity_schema_id
        if type is not UNSET:
            field_dict["type"] = type
        if api_id is not UNSET:
            field_dict["apiId"] = api_id
        if columns is not UNSET:
            field_dict["columns"] = columns
        if indentation is not UNSET:
            field_dict["indentation"] = indentation

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entity_schema_id() -> Union[Unset, str]:
            entity_schema_id = d.pop("entitySchemaId")
            return entity_schema_id

        try:
            entity_schema_id = get_entity_schema_id()
        except KeyError:
            if strict:
                raise
            entity_schema_id = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, RegistrationTableNotePartType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = RegistrationTableNotePartType(_type)
                except ValueError:
                    type = RegistrationTableNotePartType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, RegistrationTableNotePartType], UNSET)

        def get_api_id() -> Union[Unset, str]:
            api_id = d.pop("apiId")
            return api_id

        try:
            api_id = get_api_id()
        except KeyError:
            if strict:
                raise
            api_id = cast(Union[Unset, str], UNSET)

        def get_columns() -> Union[Unset, List[StructuredTableColumnInfo]]:
            columns = []
            _columns = d.pop("columns")
            for columns_item_data in _columns or []:
                columns_item = StructuredTableColumnInfo.from_dict(columns_item_data, strict=False)

                columns.append(columns_item)

            return columns

        try:
            columns = get_columns()
        except KeyError:
            if strict:
                raise
            columns = cast(Union[Unset, List[StructuredTableColumnInfo]], UNSET)

        def get_indentation() -> Union[Unset, int]:
            indentation = d.pop("indentation")
            return indentation

        try:
            indentation = get_indentation()
        except KeyError:
            if strict:
                raise
            indentation = cast(Union[Unset, int], UNSET)

        registration_table_note_part = cls(
            entity_schema_id=entity_schema_id,
            type=type,
            api_id=api_id,
            columns=columns,
            indentation=indentation,
        )

        registration_table_note_part.additional_properties = d
        return registration_table_note_part

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entity_schema_id(self) -> str:
        if isinstance(self._entity_schema_id, Unset):
            raise NotPresentError(self, "entity_schema_id")
        return self._entity_schema_id

    @entity_schema_id.setter
    def entity_schema_id(self, value: str) -> None:
        self._entity_schema_id = value

    @entity_schema_id.deleter
    def entity_schema_id(self) -> None:
        self._entity_schema_id = UNSET

    @property
    def type(self) -> RegistrationTableNotePartType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: RegistrationTableNotePartType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def api_id(self) -> str:
        if isinstance(self._api_id, Unset):
            raise NotPresentError(self, "api_id")
        return self._api_id

    @api_id.setter
    def api_id(self, value: str) -> None:
        self._api_id = value

    @api_id.deleter
    def api_id(self) -> None:
        self._api_id = UNSET

    @property
    def columns(self) -> List[StructuredTableColumnInfo]:
        if isinstance(self._columns, Unset):
            raise NotPresentError(self, "columns")
        return self._columns

    @columns.setter
    def columns(self, value: List[StructuredTableColumnInfo]) -> None:
        self._columns = value

    @columns.deleter
    def columns(self) -> None:
        self._columns = UNSET

    @property
    def indentation(self) -> int:
        """ All notes have an indentation level - the default is 0 for no indent. For lists, indentation gives notes hierarchy - a bulleted list with children is modeled as one note part with indentation 1 followed by note parts with indentation 2, for example. """
        if isinstance(self._indentation, Unset):
            raise NotPresentError(self, "indentation")
        return self._indentation

    @indentation.setter
    def indentation(self, value: int) -> None:
        self._indentation = value

    @indentation.deleter
    def indentation(self) -> None:
        self._indentation = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.custom_fields import CustomFields
from ..models.fields import Fields
from ..models.molecule_structure import MoleculeStructure
from ..types import UNSET, Unset

T = TypeVar("T", bound="MoleculeBaseRequestForCreate")


@attr.s(auto_attribs=True, repr=False)
class MoleculeBaseRequestForCreate:
    """  """

    _aliases: Union[Unset, List[str]] = UNSET
    _author_ids: Union[Unset, List[str]] = UNSET
    _chemical_structure: Union[Unset, MoleculeStructure] = UNSET
    _custom_fields: Union[Unset, CustomFields] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _folder_id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _schema_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("aliases={}".format(repr(self._aliases)))
        fields.append("author_ids={}".format(repr(self._author_ids)))
        fields.append("chemical_structure={}".format(repr(self._chemical_structure)))
        fields.append("custom_fields={}".format(repr(self._custom_fields)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("folder_id={}".format(repr(self._folder_id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "MoleculeBaseRequestForCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aliases: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._aliases, Unset):
            aliases = self._aliases

        author_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._author_ids, Unset):
            author_ids = self._author_ids

        chemical_structure: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._chemical_structure, Unset):
            chemical_structure = self._chemical_structure.to_dict()

        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._custom_fields, Unset):
            custom_fields = self._custom_fields.to_dict()

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        folder_id = self._folder_id
        name = self._name
        schema_id = self._schema_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aliases is not UNSET:
            field_dict["aliases"] = aliases
        if author_ids is not UNSET:
            field_dict["authorIds"] = author_ids
        if chemical_structure is not UNSET:
            field_dict["chemicalStructure"] = chemical_structure
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if fields is not UNSET:
            field_dict["fields"] = fields
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if name is not UNSET:
            field_dict["name"] = name
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aliases() -> Union[Unset, List[str]]:
            aliases = cast(List[str], d.pop("aliases"))

            return aliases

        try:
            aliases = get_aliases()
        except KeyError:
            if strict:
                raise
            aliases = cast(Union[Unset, List[str]], UNSET)

        def get_author_ids() -> Union[Unset, List[str]]:
            author_ids = cast(List[str], d.pop("authorIds"))

            return author_ids

        try:
            author_ids = get_author_ids()
        except KeyError:
            if strict:
                raise
            author_ids = cast(Union[Unset, List[str]], UNSET)

        def get_chemical_structure() -> Union[Unset, MoleculeStructure]:
            chemical_structure: Union[Unset, Union[Unset, MoleculeStructure]] = UNSET
            _chemical_structure = d.pop("chemicalStructure")

            if not isinstance(_chemical_structure, Unset):
                chemical_structure = MoleculeStructure.from_dict(_chemical_structure)

            return chemical_structure

        try:
            chemical_structure = get_chemical_structure()
        except KeyError:
            if strict:
                raise
            chemical_structure = cast(Union[Unset, MoleculeStructure], UNSET)

        def get_custom_fields() -> Union[Unset, CustomFields]:
            custom_fields: Union[Unset, Union[Unset, CustomFields]] = UNSET
            _custom_fields = d.pop("customFields")

            if not isinstance(_custom_fields, Unset):
                custom_fields = CustomFields.from_dict(_custom_fields)

            return custom_fields

        try:
            custom_fields = get_custom_fields()
        except KeyError:
            if strict:
                raise
            custom_fields = cast(Union[Unset, CustomFields], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_folder_id() -> Union[Unset, str]:
            folder_id = d.pop("folderId")
            return folder_id

        try:
            folder_id = get_folder_id()
        except KeyError:
            if strict:
                raise
            folder_id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_schema_id() -> Union[Unset, str]:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(Union[Unset, str], UNSET)

        molecule_base_request_for_create = cls(
            aliases=aliases,
            author_ids=author_ids,
            chemical_structure=chemical_structure,
            custom_fields=custom_fields,
            fields=fields,
            folder_id=folder_id,
            name=name,
            schema_id=schema_id,
        )

        molecule_base_request_for_create.additional_properties = d
        return molecule_base_request_for_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def aliases(self) -> List[str]:
        """ Aliases to add to the Molecule. """
        if isinstance(self._aliases, Unset):
            raise NotPresentError(self, "aliases")
        return self._aliases

    @aliases.setter
    def aliases(self, value: List[str]) -> None:
        self._aliases = value

    @aliases.deleter
    def aliases(self) -> None:
        self._aliases = UNSET

    @property
    def author_ids(self) -> List[str]:
        """ IDs of users to set as the Molecule's authors. """
        if isinstance(self._author_ids, Unset):
            raise NotPresentError(self, "author_ids")
        return self._author_ids

    @author_ids.setter
    def author_ids(self, value: List[str]) -> None:
        self._author_ids = value

    @author_ids.deleter
    def author_ids(self) -> None:
        self._author_ids = UNSET

    @property
    def chemical_structure(self) -> MoleculeStructure:
        if isinstance(self._chemical_structure, Unset):
            raise NotPresentError(self, "chemical_structure")
        return self._chemical_structure

    @chemical_structure.setter
    def chemical_structure(self, value: MoleculeStructure) -> None:
        self._chemical_structure = value

    @chemical_structure.deleter
    def chemical_structure(self) -> None:
        self._chemical_structure = UNSET

    @property
    def custom_fields(self) -> CustomFields:
        if isinstance(self._custom_fields, Unset):
            raise NotPresentError(self, "custom_fields")
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, value: CustomFields) -> None:
        self._custom_fields = value

    @custom_fields.deleter
    def custom_fields(self) -> None:
        self._custom_fields = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def folder_id(self) -> str:
        """ID of the folder containing the Molecule."""
        if isinstance(self._folder_id, Unset):
            raise NotPresentError(self, "folder_id")
        return self._folder_id

    @folder_id.setter
    def folder_id(self, value: str) -> None:
        self._folder_id = value

    @folder_id.deleter
    def folder_id(self) -> None:
        self._folder_id = UNSET

    @property
    def name(self) -> str:
        """Name of the Molecule."""
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def schema_id(self) -> str:
        """ID of the Molecule's schema."""
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @schema_id.deleter
    def schema_id(self) -> None:
        self._schema_id = UNSET



import warnings

import benchling_api_client.v2.client

Client = benchling_api_client.v2.client.Client
AuthenticatedClient = benchling_api_client.v2.client.AuthenticatedClient

warnings.warn("This package is deprecated: please use benchling_api_client.v2.client", DeprecationWarning)



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainersCheckin")


@attr.s(auto_attribs=True, repr=False)
class ContainersCheckin:
    """  """

    _container_ids: List[str]
    _comments: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("container_ids={}".format(repr(self._container_ids)))
        fields.append("comments={}".format(repr(self._comments)))
        return "ContainersCheckin({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        container_ids = self._container_ids

        comments = self._comments

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if container_ids is not UNSET:
            field_dict["containerIds"] = container_ids
        if comments is not UNSET:
            field_dict["comments"] = comments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_container_ids() -> List[str]:
            container_ids = cast(List[str], d.pop("containerIds"))

            return container_ids

        try:
            container_ids = get_container_ids()
        except KeyError:
            if strict:
                raise
            container_ids = cast(List[str], UNSET)

        def get_comments() -> Union[Unset, str]:
            comments = d.pop("comments")
            return comments

        try:
            comments = get_comments()
        except KeyError:
            if strict:
                raise
            comments = cast(Union[Unset, str], UNSET)

        containers_checkin = cls(
            container_ids=container_ids,
            comments=comments,
        )

        return containers_checkin

    @property
    def container_ids(self) -> List[str]:
        """ Array of container IDs. """
        if isinstance(self._container_ids, Unset):
            raise NotPresentError(self, "container_ids")
        return self._container_ids

    @container_ids.setter
    def container_ids(self, value: List[str]) -> None:
        self._container_ids = value

    @property
    def comments(self) -> str:
        if isinstance(self._comments, Unset):
            raise NotPresentError(self, "comments")
        return self._comments

    @comments.setter
    def comments(self, value: str) -> None:
        self._comments = value

    @comments.deleter
    def comments(self) -> None:
        self._comments = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="RegistrationOrigin")


@attr.s(auto_attribs=True, repr=False)
class RegistrationOrigin:
    """  """

    _origin_entry_id: Union[Unset, None, str] = UNSET
    _registered_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("origin_entry_id={}".format(repr(self._origin_entry_id)))
        fields.append("registered_at={}".format(repr(self._registered_at)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RegistrationOrigin({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        origin_entry_id = self._origin_entry_id
        registered_at: Union[Unset, str] = UNSET
        if not isinstance(self._registered_at, Unset):
            registered_at = self._registered_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if origin_entry_id is not UNSET:
            field_dict["originEntryId"] = origin_entry_id
        if registered_at is not UNSET:
            field_dict["registeredAt"] = registered_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_origin_entry_id() -> Union[Unset, None, str]:
            origin_entry_id = d.pop("originEntryId")
            return origin_entry_id

        try:
            origin_entry_id = get_origin_entry_id()
        except KeyError:
            if strict:
                raise
            origin_entry_id = cast(Union[Unset, None, str], UNSET)

        def get_registered_at() -> Union[Unset, datetime.datetime]:
            registered_at: Union[Unset, datetime.datetime] = UNSET
            _registered_at = d.pop("registeredAt")
            if _registered_at is not None and not isinstance(_registered_at, Unset):
                registered_at = isoparse(cast(str, _registered_at))

            return registered_at

        try:
            registered_at = get_registered_at()
        except KeyError:
            if strict:
                raise
            registered_at = cast(Union[Unset, datetime.datetime], UNSET)

        registration_origin = cls(
            origin_entry_id=origin_entry_id,
            registered_at=registered_at,
        )

        registration_origin.additional_properties = d
        return registration_origin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def origin_entry_id(self) -> Optional[str]:
        if isinstance(self._origin_entry_id, Unset):
            raise NotPresentError(self, "origin_entry_id")
        return self._origin_entry_id

    @origin_entry_id.setter
    def origin_entry_id(self, value: Optional[str]) -> None:
        self._origin_entry_id = value

    @origin_entry_id.deleter
    def origin_entry_id(self) -> None:
        self._origin_entry_id = UNSET

    @property
    def registered_at(self) -> datetime.datetime:
        if isinstance(self._registered_at, Unset):
            raise NotPresentError(self, "registered_at")
        return self._registered_at

    @registered_at.setter
    def registered_at(self, value: datetime.datetime) -> None:
        self._registered_at = value

    @registered_at.deleter
    def registered_at(self) -> None:
        self._registered_at = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError, UnknownType
from ..models.app_config_item_api_mixin_app import AppConfigItemApiMixinApp
from ..models.entity_schema_app_config_item_type import EntitySchemaAppConfigItemType
from ..models.inaccessible_resource import InaccessibleResource
from ..models.linked_app_config_resource_summary import LinkedAppConfigResourceSummary
from ..models.schema_dependency_subtypes import SchemaDependencySubtypes
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntitySchemaAppConfigItem")


@attr.s(auto_attribs=True, repr=False)
class EntitySchemaAppConfigItem:
    """  """

    _subtype: Union[Unset, SchemaDependencySubtypes] = UNSET
    _type: Union[Unset, EntitySchemaAppConfigItemType] = UNSET
    _value: Union[Unset, None, str] = UNSET
    _linked_resource: Union[
        Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
    ] = UNSET
    _description: Union[Unset, str] = UNSET
    _required_config: Union[Unset, bool] = UNSET
    _api_url: Union[Unset, str] = UNSET
    _app: Union[Unset, AppConfigItemApiMixinApp] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _path: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("subtype={}".format(repr(self._subtype)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("value={}".format(repr(self._value)))
        fields.append("linked_resource={}".format(repr(self._linked_resource)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("required_config={}".format(repr(self._required_config)))
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("app={}".format(repr(self._app)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("path={}".format(repr(self._path)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntitySchemaAppConfigItem({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        subtype: Union[Unset, int] = UNSET
        if not isinstance(self._subtype, Unset):
            subtype = self._subtype.value

        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        value = self._value
        linked_resource: Union[Unset, None, Dict[str, Any]]
        if isinstance(self._linked_resource, Unset):
            linked_resource = UNSET
        elif self._linked_resource is None:
            linked_resource = None
        elif isinstance(self._linked_resource, UnknownType):
            linked_resource = self._linked_resource.value
        elif isinstance(self._linked_resource, LinkedAppConfigResourceSummary):
            linked_resource = self._linked_resource.to_dict()

        else:
            linked_resource = self._linked_resource.to_dict()

        description = self._description
        required_config = self._required_config
        api_url = self._api_url
        app: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._app, Unset):
            app = self._app.to_dict()

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        path: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._path, Unset):
            path = self._path

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if subtype is not UNSET:
            field_dict["subtype"] = subtype
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value
        if linked_resource is not UNSET:
            field_dict["linkedResource"] = linked_resource
        if description is not UNSET:
            field_dict["description"] = description
        if required_config is not UNSET:
            field_dict["requiredConfig"] = required_config
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if app is not UNSET:
            field_dict["app"] = app
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if path is not UNSET:
            field_dict["path"] = path

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_subtype() -> Union[Unset, SchemaDependencySubtypes]:
            subtype = UNSET
            _subtype = d.pop("subtype")
            if _subtype is not None and _subtype is not UNSET:
                try:
                    subtype = SchemaDependencySubtypes(_subtype)
                except ValueError:
                    subtype = SchemaDependencySubtypes.of_unknown(_subtype)

            return subtype

        try:
            subtype = get_subtype()
        except KeyError:
            if strict:
                raise
            subtype = cast(Union[Unset, SchemaDependencySubtypes], UNSET)

        def get_type() -> Union[Unset, EntitySchemaAppConfigItemType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = EntitySchemaAppConfigItemType(_type)
                except ValueError:
                    type = EntitySchemaAppConfigItemType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, EntitySchemaAppConfigItemType], UNSET)

        def get_value() -> Union[Unset, None, str]:
            value = d.pop("value")
            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(Union[Unset, None, str], UNSET)

        def get_linked_resource() -> Union[
            Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
        ]:
            def _parse_linked_resource(
                data: Union[Unset, None, Dict[str, Any]]
            ) -> Union[Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]:
                linked_resource: Union[
                    Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType
                ]
                if data is None:
                    return data
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    linked_app_config_resource = LinkedAppConfigResourceSummary.from_dict(data, strict=True)

                    return linked_app_config_resource
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    linked_app_config_resource = InaccessibleResource.from_dict(data, strict=True)

                    return linked_app_config_resource
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            linked_resource = _parse_linked_resource(d.pop("linkedResource"))

            return linked_resource

        try:
            linked_resource = get_linked_resource()
        except KeyError:
            if strict:
                raise
            linked_resource = cast(
                Union[Unset, None, LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType], UNSET
            )

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_required_config() -> Union[Unset, bool]:
            required_config = d.pop("requiredConfig")
            return required_config

        try:
            required_config = get_required_config()
        except KeyError:
            if strict:
                raise
            required_config = cast(Union[Unset, bool], UNSET)

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_app() -> Union[Unset, AppConfigItemApiMixinApp]:
            app: Union[Unset, Union[Unset, AppConfigItemApiMixinApp]] = UNSET
            _app = d.pop("app")

            if not isinstance(_app, Unset):
                app = AppConfigItemApiMixinApp.from_dict(_app)

            return app

        try:
            app = get_app()
        except KeyError:
            if strict:
                raise
            app = cast(Union[Unset, AppConfigItemApiMixinApp], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_path() -> Union[Unset, List[str]]:
            path = cast(List[str], d.pop("path"))

            return path

        try:
            path = get_path()
        except KeyError:
            if strict:
                raise
            path = cast(Union[Unset, List[str]], UNSET)

        entity_schema_app_config_item = cls(
            subtype=subtype,
            type=type,
            value=value,
            linked_resource=linked_resource,
            description=description,
            required_config=required_config,
            api_url=api_url,
            app=app,
            created_at=created_at,
            id=id,
            modified_at=modified_at,
            path=path,
        )

        entity_schema_app_config_item.additional_properties = d
        return entity_schema_app_config_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def subtype(self) -> SchemaDependencySubtypes:
        if isinstance(self._subtype, Unset):
            raise NotPresentError(self, "subtype")
        return self._subtype

    @subtype.setter
    def subtype(self, value: SchemaDependencySubtypes) -> None:
        self._subtype = value

    @subtype.deleter
    def subtype(self) -> None:
        self._subtype = UNSET

    @property
    def type(self) -> EntitySchemaAppConfigItemType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: EntitySchemaAppConfigItemType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def value(self) -> Optional[str]:
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: Optional[str]) -> None:
        self._value = value

    @value.deleter
    def value(self) -> None:
        self._value = UNSET

    @property
    def linked_resource(
        self,
    ) -> Optional[Union[LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]]:
        if isinstance(self._linked_resource, Unset):
            raise NotPresentError(self, "linked_resource")
        return self._linked_resource

    @linked_resource.setter
    def linked_resource(
        self, value: Optional[Union[LinkedAppConfigResourceSummary, InaccessibleResource, UnknownType]]
    ) -> None:
        self._linked_resource = value

    @linked_resource.deleter
    def linked_resource(self) -> None:
        self._linked_resource = UNSET

    @property
    def description(self) -> str:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def required_config(self) -> bool:
        if isinstance(self._required_config, Unset):
            raise NotPresentError(self, "required_config")
        return self._required_config

    @required_config.setter
    def required_config(self, value: bool) -> None:
        self._required_config = value

    @required_config.deleter
    def required_config(self) -> None:
        self._required_config = UNSET

    @property
    def api_url(self) -> str:
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def app(self) -> AppConfigItemApiMixinApp:
        if isinstance(self._app, Unset):
            raise NotPresentError(self, "app")
        return self._app

    @app.setter
    def app(self, value: AppConfigItemApiMixinApp) -> None:
        self._app = value

    @app.deleter
    def app(self) -> None:
        self._app = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime the app config item was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the app config item was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def path(self) -> List[str]:
        """ Array-based representation of config item's location in the tree in order from top to bottom. """
        if isinstance(self._path, Unset):
            raise NotPresentError(self, "path")
        return self._path

    @path.setter
    def path(self, value: List[str]) -> None:
        self._path = value

    @path.deleter
    def path(self) -> None:
        self._path = UNSET
